#include <iostream>
#include <fstream>
#include <cmath>
#include <cstdlib>
#include "BG-defs.h"

using namespace std;

int main(int argc, char **argv)
{
    if(argc<5)
    {
	cerr << "Usage: " << argv[0] << " <nsteps> <HISTORY> <X (O=0, Si=1, Na=2, Ca=3, Cu=4, P=5, Cu2=6> <X-O cutoff> <Si-O cutoff>" << endl;
	return 1;
    }

    const int nsteps = atoi(argv[1]);
    const int natoms =  nO+nSi+nNa+nCa+nCu+nP+nCu2;
    const int natomss = natoms+nO; // to include shells + cores

    double poss[natomss][3], pos[natoms][3];

    int atom1, atom2;
    double dx, dy, dz, dr;

    int nbo, nnbo;

    int n[7];
    n[0] = nO;
    n[1] = nSi;
    n[2] = nNa;
    n[3] = nCa;
    n[4] = nCu;
    n[5] = nP;
    n[6] = nCu2;

    double c[7];
    for (int i=0; i<7; i++)
    {
        c[i] = double(n[i])/double(natoms);
    }

    int offset[7];
    offset[0] = 0;
    offset[1] = n[0];
    offset[2] = n[0]+n[1];
    offset[3] = n[0]+n[1]+n[2];
    offset[4] = n[0]+n[1]+n[2]+n[3];
    offset[5] = n[0]+n[1]+n[2]+n[3]+n[4];
    offset[6] = n[0]+n[1]+n[2]+n[3]+n[4]+n[5];

    const int sp1 = atoi(argv[3]);
    const double cutoff = atof(argv[4]);

    const double SiOcutoff = atof(argv[5]);
    
    const int maxnei = 16;
    int nn[n[sp1]][maxnei];

    int OSic;
    int OSicoord[nO];

    bool isbridging[nO];

    int XOc, XBOc, XNBOc;
    int XOcoord[maxnei], XBOcoord[maxnei], XNBOcoord[maxnei];
    for (int i=0; i<maxnei; i++)
    {
	XOcoord[i] = 0;
	XBOcoord[i] = 0;
	XNBOcoord[i] = 0;
    }

    char str[101];
    
    ifstream fin;
    fin.open(argv[2]);

    // get rid of top two lines
    fin.getline(str,101);
    fin.getline(str,101);

    for (int i=0; i<nsteps; i++)
    {
        if(i%100==0){cerr << i << " ";}

        // Get rid of metadata
        fin.getline(str,101);
        fin.getline(str,101);
        fin.getline(str,101);
        fin.getline(str,101);

        // Read in positions

        for (int j=0; j<natomss; j++)
        {
            fin.getline(str,101);
            fin >> poss[j][0] >> poss[j][1] >> poss[j][2];
            fin.getline(str,101);
        }

        // remove O shells
        int atom = 0;
        for (int j=0; j<natomss; j++)
        {
            if(!(j<2*nO && j%2==1)) // oxygen cores only
            {
                pos[atom][0] = poss[j][0];
                pos[atom][1] = poss[j][1];
                pos[atom][2] = poss[j][2];
                atom++;
            }
        }

	// compute X-O neighbour lists

        for (int j=0; j<n[sp1]; j++){
            nn[j][0] = 0;
            for (int k=1; k<maxnei; k++){
                nn[j][k] = -1;
            }
        }

        for (int j=0; j<n[sp1]; j++){
            for (int k=0; k<nO; k++){

                atom1 = offset[sp1]+j;
                atom2 = offset[0]+k; // oxygen

                dx = pos[atom1][0] - pos[atom2][0];
                dy = pos[atom1][1] - pos[atom2][1];
                dz = pos[atom1][2] - pos[atom2][2];

                while(dx>cell/2.0){dx-=cell;}
                while(dy>cell/2.0){dy-=cell;}
                while(dz>cell/2.0){dz-=cell;}
                while(dx<-cell/2.0){dx+=cell;}
                while(dy<-cell/2.0){dy+=cell;}
                while(dz<-cell/2.0){dz+=cell;}

                dr = sqrt(dx*dx + dy*dy + dz*dz);

                if(dr<cutoff){
                    nn[j][0]++;
                    nn[j][nn[j][0]] = atom2;
                }
            }
        }

        // construct O-Si coordination numbers

        for (int j=0; j<nO; j++){

           OSic = 0;

            for (int k=0; k<nSi; k++){
                atom1 = offset[0]+j; // oxygen
                atom2 = offset[1]+k; // silicon

                dx = pos[atom1][0] - pos[atom2][0];
                dy = pos[atom1][1] - pos[atom2][1];
                dz = pos[atom1][2] - pos[atom2][2];

                while(dx>cell/2.0){dx-=cell;}
                while(dy>cell/2.0){dy-=cell;}
                while(dz>cell/2.0){dz-=cell;}
                while(dx<-cell/2.0){dx+=cell;}
                while(dy<-cell/2.0){dy+=cell;}
                while(dz<-cell/2.0){dz+=cell;}

                dr = sqrt(dx*dx + dy*dy + dz*dz);

                if(dr<SiOcutoff){OSic++;}
            }

            OSicoord[j] = OSic;
        }

	// so is the oxygen atom bridging or not?
  
	//cout << "quick count" << endl;
        nbo = 0;
        nnbo = 0;

	for (int j=0; j<nO; j++)
	{
	    if(OSicoord[j]<2){isbridging[j]=false; nnbo++;}
	    else{isbridging[j]=true; nbo++;}
	    //cout << j << " " << OSicoord[j] << " " << OPcoord[j] << endl;
	}

	//cout << "bo: " << nbo << " nbo: " << nnbo << endl;

	// now add to the coordination arrays
	for (int j=0; j<n[sp1]; j++)
	{
	    XOc = nn[j][0];
	    XBOc = 0;
	    XNBOc = 0;
	    for (int k=1; k<=nn[j][0]; k++)
	    {
		if(isbridging[nn[j][k]]){XBOc++;}
		else{XNBOc++;}
	    }
	    // quick check
	    if(XBOc+XNBOc!=XOc)
	    {
		cerr << "ABORT! Something wrong in bridging oxygen calculations" << endl;
		return 1;
	    }

	    XOcoord[XOc]++;
	    XBOcoord[XBOc]++;
	    XNBOcoord[XNBOc]++;
	}

    }

    // and now output
    ofstream fout;
    fout.open("coords.out");
    const double dns = double(n[sp1]*nsteps);

    double tot = 0.0, BOtot = 0.0, NBOtot = 0.0;

    for (int i=0; i<maxnei; i++)
    {
	tot += i*XOcoord[i]/dns;	
	BOtot += i*XBOcoord[i]/dns;	
	NBOtot += i*XNBOcoord[i]/dns;

	fout << XOcoord[i]/dns << "\t" << XBOcoord[i]/dns << "\t" << XNBOcoord[i]/dns << endl; 
    }

    fout << tot << "\t" << BOtot << "\t" << NBOtot << endl;

    fout.close();

    return 0;
}


