#include <iostream>
#include <fstream>
#include <cmath>
#include <cstdlib>
#include "BG-defs.h"

using namespace std;

int main(int argc, char **argv)
{
    if(argc<5)
    {
	cerr << "Usage: " << argv[0] << " <nsteps> <HISTORY> <sp1> <sp2> ( O(zero) Si(1) Na(2) Ca(3) Cu(4) P(5) Cu2(6))" << endl;
	return 1;
    }
    
    int atom1, atom2, box;
    double dx, dy, dz, dr;
    char str[101];
    double inner, centre, outer;

    const int nsteps = atoi(argv[1]);
    const int natoms =  nO+nSi+nNa+nCa+nCu+nP+nCu2;
    const int natomss = natoms+nO; // to include shells + cores

    const int maxboxes = int(floor(cell/(2.0*binwidth)));

    double shellvol[maxboxes];
    int nr[maxboxes];
    double gr[maxboxes];

    double poss[natomss][3];
    double pos[natoms][3];

    const double rho = natoms/(cell*cell*cell);

    const int sp1 = atoi(argv[3]);
    const int sp2 = atoi(argv[4]);

    int n[7];
    n[0] = nO;
    n[1] = nSi;
    n[2] = nNa;
    n[3] = nCa;
    n[4] = nCu;
    n[5] = nP;
    n[6] = nCu2;

    double c[7];
    for (int i=0; i<7; i++)
    {
	c[i] = double(n[i])/double(natoms);
    }

    int offset[7];
    offset[0] = 0;
    offset[1] = n[0];
    offset[2] = n[0]+n[1];
    offset[3] = n[0]+n[1]+n[2];
    offset[4] = n[0]+n[1]+n[2]+n[3];
    offset[5] = n[0]+n[1]+n[2]+n[3]+n[4];
    offset[6] = n[0]+n[1]+n[2]+n[3]+n[4]+n[5];

    for (int i=0; i<maxboxes; i++)
    {
	inner = i*binwidth;
	centre = (i+0.5)*binwidth;
	outer = (i+1.0)*binwidth;
	
	shellvol[i] = 4.0*M_PI*(outer*outer*outer-inner*inner*inner)/3.0;
    }

    ifstream fin;
    fin.open(argv[2]);

    fin.getline(str,101); // get rid of first two lines
    fin.getline(str,101);

    // Loop round steps

    for (int j=0; j<maxboxes; j++){
	gr[j] = 0.0;
    }

    for (int i=0; i<nsteps; i++)
    {
	for (int j=0; j<maxboxes; j++){
	    nr[j] = 0;
	}
	
	if(i%100==0){cerr << i << " ";}

	// Get rid of metadata
	fin.getline(str,101);
	fin.getline(str,101);
	fin.getline(str,101);
	fin.getline(str,101);

	// Read in positions

	for (int j=0; j<natomss; j++)
	{
	    fin.getline(str,101);
	    fin >> poss[j][0] >> poss[j][1] >> poss[j][2];
	    fin.getline(str,101);
	}

	// remove O shells
	int atom = 0;
	for (int j=0; j<natomss; j++)
	{
	    if(!(j<2*nO && j%2==1)) // oxygen cores only
	    {
		pos[atom][0] = poss[j][0];
		pos[atom][1] = poss[j][1];
		pos[atom][2] = poss[j][2];
		atom++;
	    }
	}

	// gr
    
	for (int j=0; j<n[sp1]; j++){
	    for (int k=0; k<n[sp2]; k++){
		atom1 = offset[sp1]+j;
		atom2 = offset[sp2]+k;
		
		dx = pos[atom1][0] - pos[atom2][0];
		dy = pos[atom1][1] - pos[atom2][1];
		dz = pos[atom1][2] - pos[atom2][2];

		while(dx>cell/2.0){dx-=cell;}
		while(dy>cell/2.0){dy-=cell;}
		while(dz>cell/2.0){dz-=cell;}
		while(dx<-cell/2.0){dx+=cell;}
		while(dy<-cell/2.0){dy+=cell;}
		while(dz<-cell/2.0){dz+=cell;}

		dr = sqrt(dx*dx + dy*dy + dz*dz);

		box = int(floor(dr/binwidth));

		if(box<maxboxes && atom1!=atom2){nr[box]++;}
	    }
	}

	//convert into g(r)
	for (int j=0; j<maxboxes; j++)
	{
	    gr[j] += nr[j]/(shellvol[j]*natoms*c[sp1]*c[sp2]*rho);
	}


    }

    fin.close();

    // output

    ofstream fout;

    fout.open("gr.out");

    const double dnsteps = double(nsteps);
    for (int i=0; i<maxboxes; i++)
    {
	fout << (i+0.5)*binwidth << " " << gr[i]/dnsteps << endl;
    }

    fout.close();

    // compute n(r), the running integral of g(r)

    double nofr[maxboxes], r2gr[maxboxes];
    nofr[0] = 0.0;
    double running_tot = 0.0;

    for (int i=0; i<maxboxes; i++)
    {
	r2gr[i] = (i+0.5)*(i+0.5)*binwidth*binwidth*gr[i]/dnsteps;

	if(i>0)
	{
	    running_tot += 4.0*M_PI*c[sp2]*rho*0.5*(r2gr[i]+r2gr[i-1])*binwidth;
	    nofr[i] = running_tot;
	    //nofr[i] = 0.5*(r2gr[i]+r2gr[i-1])*binwidth;
	    //nofr[i] *= 4.0*M_PI*c[sp2]*rho;
	}
    }

    ofstream fout2;
    fout2.open("nr.out");
    for (int i=0; i<maxboxes; i++)
    {
	fout2 << (i+0.5)*binwidth << " " << nofr[i] << endl;
    }
    fout2.close();


    return 0;
}


